'*****************************************************
'* PowerPong 1.0                                     *
'*                                                   *
'* PowerBASIC Pong.bas                               *
'* A Pong sytle game                                 *
'* Created with PowerBASIC For Windows 9.0           *
'*                                                   *
'* Copyright (c) 2005-2008 PowerBASIC, Inc.          *
'*****************************************************

#COMPILER PBWIN 9
#COMPILE EXE
#DIM ALL

%USEMACROS = 1
#INCLUDE "WIN32API.INC"

'*****************************************************
'* User Defined Types                                *
'*****************************************************
TYPE BallType
 X              AS LONG           ' X Axis of the Ball
 Y              AS LONG           ' Y Axis of the Ball
 XVelocity      AS LONG           ' X Axis Velocity of the Ball
 YVelocity      AS LONG           ' Y Axis Velocity of the Ball
END TYPE

TYPE SpritesType
 GameWindow     AS DWORD          ' GameWindow Handle
 BackGround     AS DWORD          ' Background Bitmap Handle
 PaddleBack     AS DWORD          ' Bitmap of the Paddle
 BallBack       AS DWORD          ' Background Bitmap of the Ball
 Letters        AS DWORD
 Letter(5)      AS DWORD
END TYPE

TYPE PlayerType
 Balls          AS LONG
 Score          AS LONG
 HighName       AS STRING * 5
 OS             AS LONG
END TYPE

TYPE PaddleType
  Y             AS LONG
  YVelocity     AS LONG
END TYPE

'*****************************************************
'* Global Variables                                  *
'*****************************************************
GLOBAL hDlg        AS DWORD
GLOBAL hModule     AS DWORD
GLOBAL hTimer      AS LONG
GLOBAL Player      AS PlayerType
GLOBAL Paddle      AS PaddleType
GLOBAL Ball        AS BallType
GLOBAL Sprites     AS SpritesType
GLOBAL GameOver    AS LONG
GLOBAL Performance AS LONG
GLOBAL Letters     AS STRING
GLOBAL LetterNum   AS LONG

'*****************************************************
'* Control Identifiers                               *
'*****************************************************
%IDC_LBL_START         = 120
%IDC_GAMEWIN           = 130
%IDC_LBL_SCORE         = 160
%IDC_LBL_SCORETEXT     = 170
%IDC_LBL_BALLSTEXT     = 180
%IDC_LBL_BALLS         = 190
%IDC_LBL_HIGHSCORETEXT = 200
%IDC_LBL_HIGHSCORE     = 210

'*****************************************************
'* Constants                                         *
'*****************************************************
%MAX_X                 = 630
%MIN_X                 = 10
%MAX_Y                 = 350
%MIN_Y                 = 5
%PADDLE_WIDTH          = 15
%PADDLE_HEIGHT         = 75
%BALL_DIAM             = 20

'*****************************************************
'* Declarations                                      *
'*****************************************************
DECLARE SUB       Create_Graphics
DECLARE SUB       Destroy_Graphics
DECLARE SUB       Draw_Background
DECLARE SUB       Display_NewGameInstructions
DECLARE SUB       Initialize
DECLARE SUB       Start_Game
DECLARE SUB       Run_Game
DECLARE SUB       Move_Paddle(PaddleMove AS LONG)
DECLARE SUB       Move_Ball
DECLARE SUB       Game_Over
DECLARE SUB       Get_HighScores
DECLARE SUB       Set_HighScore
DECLARE SUB       Display_HighScores(msg AS STRING)
DECLARE SUB       Save_HighScores
DECLARE SUB       Set_Font(hFont AS LONG)
DECLARE SUB       Insert_Letter
DECLARE FUNCTION  Handle_DialogEvents AS LONG
DECLARE FUNCTION  Get_SysPerform AS LONG
DECLARE FUNCTION  Get_OSVersion AS LONG
DECLARE FUNCTION  Create_Label_Font AS LONG

'*****************************************************
'* Game Window dialog callback procedure             *
'*****************************************************
CALLBACK FUNCTION DlgProc() AS LONG
  LOCAL hFont AS LONG

  SELECT CASE CBMSG
    CASE %WM_INITDIALOG
      GameOver = 0
      Player.OS      = Get_OSVersion
      LetterNum = 0
      hTimer = 0
      Get_SysPerform                     ' Gauge system performance
      Create_Graphics                    ' Create bitmaps used in game
      Draw_Background                    ' Display the game background
      Display_HighScores("HIGH SCORES")  ' Display the high scores
      Display_NewGameInstructions        ' Display Click NEW GAME to start
      hFont = Create_Label_Font          ' Create the font used for labels
      IF hFont THEN                      ' If font was created then use it
        Set_Font(hFont)
      END IF

    CASE %WM_DESTROY
      GameOver = 2
      IF htimer THEN                     ' If Timer is still running destroy it
        KillTimer(%HWND_DESKTOP, hTimer)
        htimer = 0
      END IF
      IF hFont THEN                      ' If the font used was created succesfully
        DeleteObject hFont               ' then delete it from memory
      END IF
      Destroy_Graphics                   ' Deallocate memory used for bitmaps

    CASE %WM_COMMAND
      SELECT CASE CBCTL
        CASE %IDC_LBL_START
          IF CBCTLMSG = %BN_CLICKED THEN ' Was the NEW GAME label clicked?
            IF hTimer=0 THEN Start_Game' If not entering a high score name then start a new game
          END IF
        CASE %IDC_GAMEWIN                ' Was the graphic window clicked?
          IF LetterNum THEN              ' If entering a high score name then
            Insert_Letter                ' insert the character clicked
          END IF
      END SELECT

  END SELECT
END FUNCTION

'*****************************************************
'* PBMAIN                                            *
'*                                                   *
'* Program Entry Point                               *
'*****************************************************
FUNCTION PBMAIN () AS LONG
  DIM HighScores(1 TO 5) AS GLOBAL LONG
  DIM HighNames(1 TO 5) AS GLOBAL STRING * 5

  ' Retreive the high scores from disk
  Get_HighScores

  DIALOG  FONT  "MS SANS SERIF", 12
  DIALOG  NEW PIXELS, 0, "PowerBASIC Pong",,, 640, 480, %WS_SYSMENU OR %WS_CAPTION, 0 TO hDlg

  DIALOG  SET COLOR  hdlg, %BLUE, RGB(0, 51, 102)

  CONTROL ADD LABEL, hdlg, %IDC_LBL_START, "NEW GAME", 5, 0, 100, 20, %SS_NOTIFY
  CONTROL SET COLOR  hdlg, %IDC_LBL_START, %GREEN, RGB(0, 51, 102)

  CONTROL ADD LABEL, hdlg, %IDC_LBL_HIGHSCORETEXT, "HIGH SCORE", 225, 0, 150, 20

  CONTROL SET COLOR  hdlg, %IDC_LBL_HIGHSCORETEXT, RGB(38, 124, 148), RGB(0, 51, 102)

  CONTROL ADD LABEL, hdlg, %IDC_LBL_HIGHSCORE, FORMAT$(HighScores(1), "0000"), _
                     350, 0, 80, 20, %SS_RIGHT
  CONTROL SET COLOR  hdlg, %IDC_LBL_HIGHSCORE, %RED, RGB(0, 51, 102)

  CONTROL ADD LABEL, hdlg, %IDC_LBL_SCORETEXT, "SCORE", 460, 0, 80, 20
  CONTROL SET COLOR  hdlg, %IDC_LBL_SCORETEXT, RGB(38, 124, 148), RGB(0, 51, 102)

  CONTROL ADD LABEL, hdlg, %IDC_LBL_SCORE, "0000", 585, 0, 45, 20, %SS_RIGHT
  CONTROL SET COLOR  hdlg, %IDC_LBL_SCORE, %RED, RGB(0, 51, 102)

  CONTROL ADD LABEL, hdlg, %IDC_LBL_BALLSTEXT, "BALLS LEFT", 460, 25, 100, 20
  CONTROL SET COLOR  hdlg, %IDC_LBL_BALLSTEXT, RGB(38, 124, 148), RGB(0, 51, 102)

  CONTROL ADD LABEL, hdlg, %IDC_LBL_BALLS, "3", 610, 25, 22, 20, %SS_RIGHT
  CONTROL SET COLOR  hdlg, %IDC_LBL_BALLS, %RED, RGB(0, 51, 102)

  ' Add a static graphic window to the dialog box. The %SS_NOTIFY style will
  ' notify the callback routine when the graphic window is clicked with the mouse
  CONTROL ADD GRAPHIC, hdlg, %IDC_GAMEWIN, "", 0, 50, 640, 430, _
                      %WS_CHILD OR %WS_VISIBLE OR %SS_NOTIFY, %WS_EX_CLIENTEDGE

  ' Store the handle to the graphics window in Sprites.GameWindow
  Sprites.GameWindow = getDlgItem(hdlg, %IDC_GAMEWIN)

  DIALOG  SHOW MODAL hDlg, CALL DlgProc

  PlaySound("AppStart.wav", BYVAL hModule, %SND_FILENAME OR %SND_ASYNC)
END FUNCTION

'*****************************************************
'* Create_Graphics                                   *
'*                                                   *
'* Create all the bitmaps used in the game           *
'*****************************************************
SUB Create_Graphics
  LOCAL x AS LONG
  LOCAL y AS LONG
  LOCAL c AS LONG

  ' Create a memory bitamp and store the handle in Sprites.PaddleBack
  ' This bitmap is used to store the background behind the paddle
  GRAPHIC BITMAP NEW %PADDLE_WIDTH, %PADDLE_HEIGHT TO Sprites.PaddleBack

  ' Select the paddle background as the current graphics window
  GRAPHIC ATTACH Sprites.PaddleBack, 0

  ' Clear the current graphics window with the color black
  GRAPHIC CLEAR %BLACK

  ' Create a memory bitamp and store the handle in Sprites.BallBack
  ' This bitmap is used to store the background behind the ball
  GRAPHIC BITMAP NEW %BALL_DIAM, %BALL_DIAM TO Sprites.BallBack

  ' Select the Ball background as the current graphics window
  GRAPHIC ATTACH Sprites.BallBack, 0

  ' Clear the current graphics window with the color black
  GRAPHIC CLEAR %BLACK

  ' Create a memory bitmap and load the image backg.bmp into it
  ' Store the handle to this memory bitmap in Sprites.Background
  ' This bitmap is used to store the background game image
  GRAPHIC BITMAP LOAD "backg.bmp", 640, 480 TO Sprites.Background

  ' Create a new memory bitmap and store the handle in Sprites.Letters
  ' This bitmap is used to store characters that are displayed when
  ' a player gets a high score and enters there name
  GRAPHIC BITMAP NEW 600, 300 TO Sprites.Letters

  ' Select Sprites.Letters as the current graphics window
  GRAPHIC ATTACH Sprites.Letters, 0

  ' Select the Foreground color of the current graphics window to green
  ' and the background color to transparent
  GRAPHIC COLOR %GREEN, -2

  ' Clear the current graphics window with the color black
  ' This prevents a washed out look on some video cards
  GRAPHIC CLEAR %BLACK

  ' Set the font for the current graphics window
  GRAPHIC FONT "Courier New", 25, 1

  ' List of the characters available when entering in a players name
  Letters = "ABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890!@#$%^&*()-=+,.<>/?;:[]{}"
  x = 0
  y = -40
  c= 1

  DO
    IF (x MOD 13) = 0 THEN
      y = y + 40
      x = 0
    END IF

    ' Set the coordinates where the next drawing will occur
    GRAPHIC SET POS (x*40+6, y)

    ' Print a character to the current graphics window
    GRAPHIC PRINT MID$(Letters, c, 1)

    INCR c
    INCR x
  LOOP UNTIL c>LEN(Letters)

  y = y + 20

  ' Draw a left arrow to the current graphics window
  ' Set the Graphic Line Width to 2
  GRAPHIC WIDTH 2

  ' Draw the lines for the arrow head
  GRAPHIC LINE (x*40+8, y) - (x*40+18, y-7)
  GRAPHIC LINE (x*40+8, y) - (x*40+18, y+7)

  ' The body of the arrow just looked better being a width of 3
  GRAPHIC WIDTH 3

 ' Draw the line for the arrow body
  GRAPHIC LINE (x*40+8, y) - (x*40+30, y)

  INCR x

  ' Draw a right arrow to the current graphics window
  ' Set the Graphic Line Width to 2
  GRAPHIC WIDTH 2

  ' Draw the lines for the arrow head
  GRAPHIC LINE (x*40+30, y)  - (x*40+20, y-7)
  GRAPHIC LINE (x*40+30, y)  - (x*40+20, y+7)

  ' The body of the arrow just looked better being a width of 3
  GRAPHIC WIDTH 3

  ' Draw the line for the arrow body
  GRAPHIC LINE (x*40+8, y) - (x*40+30, y)

  INCR x

  ' Set the coordinates where the next drawing will occur
  GRAPHIC SET POS (x*40+6, y)

  ' Set the font to use with the current graphics window
  GRAPHIC FONT "Courier New", 10, 1

  ' Print the text DEL (delete) to the current graphic window
  GRAPHIC PRINT "DEL"

  INCR x

  ' Set the coordinates where the next drawing will occur
  GRAPHIC SET POS (x*40+6, y)

  ' Print the text END to the current graphic window
  GRAPHIC PRINT "END"

  ' Create the bitmaps used to enter the players name
  FOR x = 1 TO 5
    GRAPHIC BITMAP NEW 40, 40 TO Sprites.Letter(x)
  NEXT x
END SUB

'*****************************************************
'* Destroy_Graphics                                  *
'*                                                   *
'* Destroy all the bitmaps used in the game.         *
'* Bitmaps created with GRAPHIC BITMAP NEW must be   *
'* closed with GRAPHIC BITMAP END before there       *
'* memory is deallocated.                            *
'*****************************************************
SUB Destroy_Graphics
  LOCAL i AS LONG

  ' Destroy the bitmaps used for each letter in a players name
  FOR i = 1 TO 5
    GRAPHIC ATTACH Sprites.Letter(i), 0
    GRAPHIC BITMAP END
  NEXT i

  ' Destroy the bitmap used to store the background behind the paddle
  GRAPHIC ATTACH Sprites.PaddleBack, 0
  GRAPHIC BITMAP END

  ' Destory the bitmap used to store the background behind the ball
  GRAPHIC ATTACH Sprites.BallBack, 0
  GRAPHIC BITMAP END

  ' Destory the bitmap used to store the background
  GRAPHIC ATTACH Sprites.Background, 0
  GRAPHIC BITMAP END
END SUB

'*****************************************************
'* Display_NewGameInstructions                       *
'*                                                   *
'* Displays instructions on how to start the game    *
'*****************************************************
SUB Display_NewGameInstructions
  LOCAL msg AS STRING
  LOCAL w   AS LONG
  LOCAL h   AS LONG

  msg = "Click NEW GAME to start"

  ' Get the width and height of the text to be displayed
  GRAPHIC TEXT SIZE msg TO w, h

  ' Set the coordinates where the next drawing will occur
  GRAPHIC SET POS ((640 - w)/2, 300)

  ' Set the foreground color to WHITE and the background to transparent
  GRAPHIC COLOR %WHITE, -2

  ' Print to the currently selected graphics window
  GRAPHIC PRINT msg
END SUB

'*****************************************************
'* Draw_Background                                   *
'*                                                   *
'* Copy the background bitmap to the game window.    *
'*****************************************************
SUB Draw_Background
  ' Make the game window the currently attached graphic window
  GRAPHIC ATTACH Sprites.GameWindow, 0

  ' Copy the background bitmap to the currently attached graphic window
  GRAPHIC COPY Sprites.Background, 0
END SUB

'*****************************************************
'* Create_Label_Font                                 *
'*                                                   *
'* Create the font used for labels on dialog.        *
'*****************************************************
FUNCTION Create_Label_Font AS LONG
  LOCAL f AS LOGFONT

  f.lfHeight          = 20
  f.lfWidth           = 0
  f.lfEscapement      = 0
  f.lfOrientation     = %dmorient_portrait
  f.lfWeight          = %FW_BOLD
  f.lfItalic          = 0
  f.lfUnderline       = 0
  f.lfStrikeOut       = 0
  f.lfCharSet         = %DEFAULT_CHARSET
  f.lfOutPrecision    = %OUT_DEFAULT_PRECIS
  f.lfClipPrecision   = %CLIP_DEFAULT_PRECIS
  f.lfQuality         = %DEFAULT_QUALITY
  f.lfPitchAndFamily  = %DEFAULT_PITCH
  f.lfFaceName        = "Arial"

  Create_Label_Font = CreateFontIndirect(f)
END FUNCTION

'*****************************************************
'* Set_Font                                          *
'*                                                   *
'* Set the font to be used for labels on the dialog. *
'*                                                   *
'* Arugements:                                       *
'*    hFont: LONG integer used to store the font     *
'*****************************************************
SUB Set_Font(hFont AS LONG)
  CONTROL SEND hdlg, %IDC_LBL_START, %WM_SETFONT, hFont, 0
  CONTROL SEND hdlg, %IDC_LBL_HIGHSCORETEXT, %WM_SETFONT, hFont, 0
  CONTROL SEND hdlg, %IDC_LBL_HIGHSCORE, %WM_SETFONT, hFont, 0
  CONTROL SEND hdlg, %IDC_LBL_SCORE, %WM_SETFONT, hFont, 0
  CONTROL SEND hdlg, %IDC_LBL_SCORETEXT, %WM_SETFONT, hFont, 0
  CONTROL SEND hdlg, %IDC_LBL_BALLS, %WM_SETFONT, hFont, 0
  CONTROL SEND hdlg, %IDC_LBL_BALLSTEXT, %WM_SETFONT, hFont, 0
END SUB

'*****************************************************
'* Move_Paddle                                       *
'*                                                   *
'* Replace the paddle with the background image that *
'* is behind it and then draw the paddle at it's new *
'* coordinates.                                      *
'*                                                   *
'* Arugements:                                       *
'*    PaddleMove: LONG integer used to store the y   *
'*                axis of the paddles new location   *
'*****************************************************
SUB Move_Paddle(PaddleMove AS LONG)
  ' Set the Game Window as the current graphics window
  ' Use the REDRAW parameter to buffer all graphics
  ' until a GRAPHIC REDRAW statement is called.
  GRAPHIC ATTACH hdlg, %IDC_GAMEWIN, REDRAW

  ' Replace the paddle with the background image behind it
  GRAPHIC COPY Sprites.PaddleBack, 0 TO (%MIN_X, Paddle.Y)

  ' Move the Paddle to its new location
  Paddle.Y = PaddleMove
  IF Paddle.Y>%MAX_Y THEN
    Paddle.Y= %MAX_Y
  ELSE
    IF Paddle.Y<%MIN_Y THEN
      Paddle.Y = %MIN_Y
    END IF
  END IF

  ' Set the paddle background bitmap as the current graphics window
  GRAPHIC ATTACH Sprites.PaddleBack, 0

  ' Copy the background at the new paddles location to the paddle's
  ' background bitmap
  GRAPHIC COPY Sprites.GameWindow, 0, _
               (%MIN_X, Paddle.Y) - (25, Paddle.Y+%PADDLE_HEIGHT) TO (0, 0)

  ' Set the game window as the current graphics window
  GRAPHIC ATTACH hDlg, %IDC_GAMEWIN, REDRAW

  ' Draw a rounded rectangle for the paddle onto the graphics window
  GRAPHIC BOX (%MIN_X, Paddle.Y) - (25, Paddle.Y+%PADDLE_HEIGHT), 40, %WHITE, %WHITE

  ' Draw all buffered graphics to the current graphic window
  GRAPHIC REDRAW
END SUB

'*****************************************************
'* Move_Ball                                         *
'*                                                   *
'* Replace the ball with the background image that   *
'* is behind it and then draw the ball at it's new   *
'* coordinates.                                      *
'*****************************************************
SUB Move_Ball
  STATIC hits AS LONG

  ' Set the game window as the current graphics window
  ' and buffer all drawing statements
  GRAPHIC ATTACH hdlg, %IDC_GAMEWIN, REDRAW

  ' Replace the ball with the background image behind it
  GRAPHIC COPY Sprites.BallBack, 0 TO (Ball.X, Ball.Y)

  ' Move ball to new location
  Ball.X = Ball.X + Ball.XVelocity
  Ball.Y = Ball.Y + Ball.YVelocity


  ' If the ball went passed the paddles X coordinate because
  ' of a high XVelocity calculate where the ball would of been
  ' right at the point of impact with the paddle.
  IF Ball.X < 26 THEN
    Ball.Y = Ball.Y + (Ball.YVelocity * ((26 - Ball.X) / Ball.XVelocity))
    Ball.X = 26
  END IF

  ' Set the balls background as the current graphics window
  GRAPHIC ATTACH Sprites.BallBack, 0

  ' Copy the background of the balls new location to balls
  ' background bitmap
  GRAPHIC COPY Sprites.GameWindow, 0, _
               (Ball.X, Ball.Y) - (Ball.X +%BALL_DIAM, Ball.Y + %BALL_DIAM) TO (0, 0)

  ' Set the game window as the current graphics window and
  ' buffer all drawing statements
  GRAPHIC ATTACH hDlg, %IDC_GAMEWIN, REDRAW

  ' Draw a solid circle for the ball at it's new location
  GRAPHIC ELLIPSE (Ball.X, Ball.Y) - (Ball.X + %BALL_DIAM, Ball.Y + %BALL_DIAM), _
                   %WHITE, %WHITE

  '******************************************
  '* Collision Detection                    *
  '******************************************
   IF Ball.X = 26 THEN
      IF (Ball.Y + 23 > Paddle.Y) AND (Ball.Y < Paddle.Y+80) THEN
        '******************************************
        '* Ball hit the paddle                    *
        '******************************************

        ' Increase the balls speed every 5 hits of the paddle
        INCR hits
        IF hits MOD 5 = 0 THEN DECR Ball.XVelocity


        ' Make sure the ball is not going too fast
        IF Ball.XVelocity < -12 THEN Ball.XVelocity = -12

        ' Send the ball back the other direction (away from the paddle)
        Ball.XVelocity = - Ball.XVelocity

        ' Calculate the YVelocity based on where the ball hit the paddle
        Ball.YVelocity = ((Ball.Y+10) - (Paddle.Y + 37))/7

        ' If the ball hit the center of the paddle make sure that the
        ' ball will not stay with YVelocity of zero forever
        IF Ball.YVelocity=0 THEN Ball.YVelocity = RND (-2, 2)

        ' Increase score every time the ball hits the paddle
        Player.Score = Player.Score + 10

        ' Display new players score
        CONTROL SET TEXT hdlg, %IDC_LBL_SCORE, FORMAT$(Player.Score, "0000")

        ' Play a "boink" sound when the ball hits the paddle
        PlaySound("BOINK1.wav", BYVAL hModule, %SND_FILENAME OR %SND_ASYNC)

      ELSE
        '******************************************
        '* Missed Ball                            *
        '******************************************

        ' Draw all buffered graphics to the current graphic window
        ' This is to show the ball missing the paddle
        GRAPHIC REDRAW

        ' Undraw the ball
        GRAPHIC COPY Sprites.BallBack, 0 TO (Ball.X, Ball.Y)
        GRAPHIC REDRAW

        ' Start the next ball at the back edge of the game window
        Ball.X = 600
        Ball.Y = 125

        ' Give the ball a random angle to start with
        Ball.YVelocity = RND(-3, 3)

        ' Play a sound indicating the player missed the ball
        PlaySound("MissedBall.wav", BYVAL hModule, %SND_FILENAME OR %SND_ASYNC)

        ' Move paddle to center of the game window
        'Move_Paddle(178)

        ' Wait before sending out a new ball
        SLEEP 1500

        ' Decrement the number of balls left
        DECR Player.Balls

        ' If no balls left then end the game
        IF Player.Balls = -1 THEN
          GameOver = 1
          EXIT SUB
        ELSE
          ' Display Number of Balls Left
          CONTROL SET TEXT hdlg, %IDC_LBL_BALLS, LTRIM$(STR$(Player.Balls))
        END IF
    END IF  ' End paddle collision detection
  ELSE
    '******************************************
    '* Detect Side Collision                  *
    '******************************************
    IF (Ball.Y < 0) THEN
      ' Ball has hit the top of the game window

      Ball.YVelocity = - Ball.YVelocity
      Ball.Y = 0
      PlaySound("BOINK1.wav", BYVAL hModule, %SND_FILENAME OR %SND_ASYNC)
    ELSE
      IF (Ball.Y > 400) THEN
        ' Ball has hit the bottom of the game window

        GRAPHIC COPY Sprites.BallBack, 0 TO (Ball.X, Ball.Y)

        Ball.YVelocity = - Ball.YVelocity
        Ball.Y = 400
        PlaySound("BOINK1.wav", BYVAL hModule, %SND_FILENAME OR %SND_ASYNC)
      ELSE
        '******************************************
        '* Detect Back Collision                  *
        '******************************************
        IF Ball.X > 620 THEN
          ' Ball has hit the back of the game window
          Ball.XVelocity = - Ball.XVelocity
          Ball.X = 620
          PlaySound("BOINK1.wav", BYVAL hModule, %SND_FILENAME OR %SND_ASYNC)
        ELSE
          'Back_Collision = %FALSE
        END IF  ' End Back Collision
      END IF    ' End Bottom Collision
    END IF      ' End Top Collision
  END IF        ' End Collison Detection

END SUB

'*****************************************************
'* Game_Over                                         *
'*                                                   *
'* The game is over. Check to see if the player has  *
'* acheived a high score. If player has a new high   *
'* score then get the players name and save it to    *
'* disk with all the other high scores.              *
'*****************************************************
SUB Game_Over
  LOCAL w    AS LONG
  LOCAL h    AS LONG
  LOCAL i    AS LONG
  LOCAL msg  AS STRING
  LOCAL High AS LONG

  ' If player closed the window exit immediatly
  IF GameOver = 2 THEN EXIT SUB

  ' Check to see if the player got a high score
  i = 0
  High = 0
  DO
    INCR i
    IF Player.Score > HighScores(i) THEN High = i
  LOOP UNTIL (i=5) OR (High>0)

  IF High THEN
    ' Play a sound indicating a new high score
    PlaySound("Winner.wav", BYVAL hModule, %SND_FILENAME OR %SND_ASYNC)

    ' Get the players name
    Set_HighScore

    ' Insert the players name into the high score list
    ARRAY INSERT HighScores(High), Player.Score

    ' Insert the players score into the high score list
    ARRAY INSERT HighNames(High), Player.HighName

    ' Save the high scores to disk
    Save_HighScores

    ' Display the current highest score
    CONTROL SET TEXT hdlg, %IDC_LBL_HIGHSCORE, FORMAT$(HighScores(1), "0000")

    msg = "HIGH SCORE"
  ELSE
    ' Play a sound indicating the player did not get a high score
    PlaySound("loser.wav", BYVAL hModule, %SND_FILENAME OR %SND_ASYNC)

    msg = "GAME OVER"
  END IF

  ' Display the current high scores
  Display_HighScores(msg)

  ' Display Instructions on how to start a new game
  Display_NewGameInstructions

  ' Draw all buffered graphics to the current graphic window
  GRAPHIC REDRAW
END SUB

'*****************************************************
'* Display_HighScores                                *
'*                                                   *
'* Display all the high scores to the game window    *
'*                                                   *
'* Arugements:                                       *
'*    msg: STRING used to store message to display   *
'*         above the high score list.                *
'*****************************************************
SUB Display_HighScores(msg AS STRING)
  LOCAL i AS LONG
  LOCAL w AS LONG
  LOCAL h AS LONG

  ' Set the game window as the current graphics window
  ' and buffer all drawing statements
  GRAPHIC ATTACH hDlg, %IDC_GAMEWIN, REDRAW

  ' Set the font for the current graphics window
  GRAPHIC FONT "Courier New", 42, 1

  ' Get the chracter size for the graphics window font
  GRAPHIC CHR SIZE TO w, h

  ' Calculate the width in pixels of the message
  w = (w * LEN(msg))

  ' Set the coordinates of the next drawing statement
  ' to center the message on the graphics window
  GRAPHIC SET POS ((640 - w) / 2, 25)

  ' Set the foreground color to RED and the background to transparent
  GRAPHIC COLOR %RED, -2

  ' Print the message to the current graphics window
  GRAPHIC PRINT msg

  ' Set the font for the current graphics window
  GRAPHIC FONT "Courier New", 25, 1

  ' Set the foreground color to GREEN and the background to transparent
  GRAPHIC COLOR %GREEN, -2

  ' Get the chracter size for the graphics window font
  GRAPHIC CHR SIZE TO w, h

  ' Calculate the width in pixels of the high score name
  ' plus 10 spaces and the high score
  w = (640-(w * 19)) / 2

  ' Display the 5 top high scores
  FOR i=1 TO 5
    ' Set the coordinates of the next drawing statement
    GRAPHIC SET POS (w, i*25+60)

    ' If there is no high score display EMPTY for the name
    ' a 0000 for the score
    IF HighNames(i)=SPACE$(5) AND HighScores(i)=0 THEN
      GRAPHIC PRINT "EMPTY" + SPACE$(10)+"0000"
    ELSE
      ' Display the high score name and high score
      GRAPHIC PRINT HighNames(i) + SPACE$(10) + FORMAT$(HighScores(i), "0000")
    END IF
  NEXT i

  ' Draw all buffered graphics to the current graphic window
  GRAPHIC REDRAW
END SUB

'*****************************************************
'* Get_HighScores                                    *
'*                                                   *
'* Read the high score names and scores from the disk*
'*****************************************************
SUB Get_HighScores
  LOCAL buff AS STRING * 4
  LOCAL i AS LONG

  i = 0
  OPEN "Pong.dat" FOR BINARY AS #1

  FOR i = 1 TO 5
    GET$ #1, 5, HighNames(i)
    GET$ #1, 4, buff
    HighScores(i) = VAL(buff)
  NEXT i
  CLOSE #1

  ' Sort the high scores in descending order
  ' The scores should already be in descending order
  ' but this is just in case somebody edited the high
  ' score file
  ARRAY SORT HighScores(), TAGARRAY HighNames(), DESCEND
END SUB

'*****************************************************
'* Set_HighScore                                     *
'*                                                   *
'* Get the players name that acheived the high score *
'*****************************************************
SUB Set_HighScore
  LOCAL w AS LONG
  LOCAL h AS LONG
  LOCAL x AS LONG

  Player.HighName = "    "

  ' Set the foreground color to RED and the background to transparent
  GRAPHIC COLOR %RED, -2

  ' Set the current graphics window font
  GRAPHIC FONT "MS San Serif", 20, 1

  ' Get the text size of "NEW HIGH SCORE"
  GRAPHIC TEXT SIZE "NEW HIGH SCORE" TO w, h

  ' Set the coordinates of the next drawing statement
  GRAPHIC SET POS ((640-w)/2, 2)

  ' Print "NEW HIGH SCORE" to the graphics window
  GRAPHIC PRINT "NEW HIGH SCORE"

  ' Get the text size of the text "0000"
  GRAPHIC TEXT SIZE "0000" TO w, h

  ' Set the coordinates of the next drawing statement
  GRAPHIC SET POS ((640-w)/2, 50)

  ' Print the players score to the graphics window
  GRAPHIC PRINT FORMAT$(Player.Score, "0000")

  ' Set the current graphics window font
  GRAPHIC FONT "MS San Serif", 10, 1

  ' Get the text size of "Please Enter Your Name"
  GRAPHIC TEXT SIZE "Please Enter Your Name" TO w, h

  ' Set the coordinates of the next drawing statement
  GRAPHIC SET POS ((640-w)/2, 100)

  ' Print "Please Enter Your Name" to the graphics window
  GRAPHIC PRINT "Please Enter Your Name"

  ' Set the foreground color to WHITE and the background to transparent
  GRAPHIC COLOR %WHITE, -2

  ' Set the Graphic Line Width to 2
  GRAPHIC WIDTH 2

  ' Draw the lines used to show place holders for all five
  ' characters in the players name
  GRAPHIC LINE (200, 175) - (240, 175)
  GRAPHIC LINE (250, 175) - (290, 175)
  GRAPHIC LINE (300, 175) - (340, 175)
  GRAPHIC LINE (350, 175) - (390, 175)
  GRAPHIC LINE (400, 175) - (440, 175)

  ' Copy the background where the the characters name
  ' will be displayed. This is to allow editing of
  ' the five characters in the name
  FOR x = 1 TO 5
    ' Set Sprites.Letter(x) as the current graphics window
    GRAPHIC ATTACH Sprites.Letter(x), 0

    ' Copy the games background where the characters in the players
    ' name will be displayed into the currently selected graphics window
    GRAPHIC COPY Sprites.GameWindow, %IDC_GAMEWIN, (150+x*50, 135) - (150+x*50+40, 175) TO (0, 0)
  NEXT x

  ' Set the game window as the currently selected graphics window
  GRAPHIC ATTACH hdlg, %IDC_GAMEWIN

  ' Copy the bitmap of characters that are used to enter the players
  ' name to the currently selected graphics window
  GRAPHIC COPY Sprites.Letters, 0 TO (50, 210), %MIX_MERGESRC

  ' Set the font for the graphics window
  GRAPHIC FONT "Courier New", 25, 1

  ' Set the foreground color to RED and the background to transparent
  GRAPHIC COLOR %RED, -2

  LetterNum = 1

  ' Create a timer that will flash the current letter place holder
  hTimer = SetTimer(%HWND_DESKTOP, 1, 200, CODEPTR(TimerProc))

  ' Process pending window or dialog messages until the player
  ' is done entering there name
  DO
    DIALOG DOEVENTS
  LOOP UNTIL hTimer=0

  ' Draw the games background
  Draw_Background
END SUB

'*****************************************************
'* Save_HighScore                                    *
'*                                                   *
'* Save the high scores and names to disk            *
'*****************************************************
SUB Save_HighScores
  LOCAL i AS LONG

  OPEN "Pong.dat" FOR BINARY AS #1

  FOR i = 1 TO 5
    PUT$ #1, HighNames(i)
    PUT$ #1, FORMAT$(HighScores(i), "0000")
  NEXT i

  CLOSE #1
END SUB

'*****************************************************
'* TimerProc                                         *
'*                                                   *
'* Flash the players name place holders white and red*
'* to let the player now what the current location is*
'*                                                   *
'* Parameters                                        *
'*    hwnd:    The window associated with the timer  *
'*    wMsg:    %WM_TIMER message                     *
'*    idEvent: The timer's identifier                *
'*    dwTime:  Number of milliseconds to run timer   *
'*****************************************************
FUNCTION TimerProc(BYVAL hwnd AS LONG, BYVAL wMsg AS LONG, BYVAL idEvent AS DWORD PTR, BYVAL dwTime AS DWORD) AS LONG
  STATIC clr AS LONG

  clr = IIF&(clr=%WHITE, %RED, %WHITE)

  ' Set the foreground color to clr and the background to transparent
  GRAPHIC COLOR clr, -2

  ' Set the width of the place holder line to 2
  GRAPHIC WIDTH 2

  ' Draw the place holder line
  GRAPHIC LINE (150+LetterNum*50, 175) - (150+LetterNum*50+40, 175)

  TimerProc = %True
END FUNCTION

'*****************************************************
'* Insert_Letter                                     *
'*                                                   *
'* Insert a character into the high score players    *
'* name. This routine is called when the player      *
'* clicks on the game window and is also entering    *
'* there name.                                       *
'*****************************************************
SUB Insert_Letter
  LOCAL CursorPos AS POINTAPI
  LOCAL x         AS LONG
  LOCAL y         AS LONG

  GetCursorPos(CursorPos)
  ScreenToClient(Sprites.GameWindow, CursorPos)

  ' Did the player click on a character?
  ' If not then exit this subroutine
  IF (CursorPos.x<40)  OR (CursorPos.x>565) OR _
     (CursorPos.y<210) OR (CursorPos.y>510) THEN EXIT SUB

  ' Calculate which character the player clicked on
  x = FIX((CursorPos.x - 50)/40) + 1
  y = FIX((CursorPos.y - 210)/40)
  x = x + (y*13)

  ' Set the foreground color to WHITE and the background to transparent
  GRAPHIC COLOR %WHITE, -2

  ' Set the drawing width to 2
  GRAPHIC WIDTH 2

  ' Draw the character place holder
  ' This is to make sure that it is always white and not red
  GRAPHIC LINE (150+LetterNum*50, 175) - (150+LetterNum*50+40, 175)

  SELECT CASE x
    CASE <62              ' Standard character clicked
      ' Copy the background image stored for this position in the name
      ' This incase the user backs up to change this letter
      GRAPHIC COPY Sprites.Letter(LetterNum),0 TO (150+LetterNum*50, 135)

      ' Set the drawing coordinates to the current position in the name
      GRAPHIC SET POS (150+LetterNum*50+6, 135)

      ' Print the letter the player clicked on
      GRAPHIC PRINT MID$(Letters, x, 1)

      ' Add this character to the players name
      Player.HighName = LEFT$(Player.HighName, LetterNum-1) _
                      + MID$(Letters, x, 1) _
                      + MID$(Player.HighName, LetterNum+1, LEN(Player.HighName))

      INCR LetterNum

    CASE 62               ' Left Arrow clicked
      DECR LetterNum

    CASE 63               ' Right Arrow clicked
      INCR LetterNum

    CASE 64               ' DEL (delete) text clicked
      GRAPHIC COPY Sprites.Letter(LetterNum),0 TO (150+LetterNum*50, 135)

      Player.HighName = LEFT$(Player.HighName, LetterNum-1) _
                      + " " _
                      + MID$(Player.HighName, LetterNum+1, LEN(Player.HighName))

    CASE 65               ' END text clicked
      KillTimer(%HWND_DESKTOP, hTimer)
      hTimer = 0

  END SELECT

  IF LetterNum<1 THEN
    LetterNum = 1
  ELSEIF LetterNum>5 THEN
    letterNum = 5
  END IF

END SUB


'*****************************************************
'* Initialize                                        *
'*                                                   *
'* Initialize variables and bitmaps used in the game.*
'*****************************************************
SUB Initialize
  RANDOMIZE TIMER

  Paddle.Y       = 125
  Ball.X         = 620
  Ball.Y         = 125
  Ball.XVelocity = -5
  Ball.YVelocity = RND(-5, 5)
  Player.Balls   = 2
  Player.Score   = 0
  GameOver       = 0
  LetterNum      = 0

  ' Set the graphic drawing width to 1
  GRAPHIC WIDTH 1

  ' Draw the games background
  Draw_Background

  ' Set the players score to 0 and balls left to 3
  CONTROL SET TEXT hdlg, %IDC_LBL_SCORE, FORMAT$(Player.Score, "0000")
  CONTROL SET TEXT hdlg, %IDC_LBL_BALLS, LTRIM$(STR$(Player.Balls))

  ' Set the paddles background as the current graphics window
  GRAPHIC ATTACH Sprites.PaddleBack, 0

  ' Copy the background behind the paddle the paddles background bitmap
  GRAPHIC COPY Sprites.GameWindow, 0, _
               (%MIN_X, Paddle.Y) -  (%MIN_X+%PADDLE_WIDTH, Paddle.Y+%PADDLE_HEIGHT) TO (0, 0)


  ' Set the balls background as the current graphics window
  GRAPHIC ATTACH Sprites.BallBack, 0

  ' Copy the background behind the ball the balls background bitmap
  GRAPHIC COPY Sprites.GameWindow, 0, _
               (Ball.X, Ball.Y) - (Ball.X + %BALL_DIAM, Ball.Y + %BALL_DIAM) TO (0, 0)
END SUB

'*****************************************************
'* Start_Game                                        *
'*                                                   *
'* Start the game.                                   *
'*****************************************************
SUB Start_Game
  PlaySound("GameStart.wav", BYVAL hModule, %SND_FILENAME)
  Initialize

  Run_Game
  Game_Over
END SUB

'*****************************************************
'* Run_Game                                          *
'*                                                   *
'* The main game loop.                               *
'*****************************************************
SUB Run_Game
  LOCAL iTime AS LONG
  LOCAL p     AS POINTAPI

  DO
    iTime = GetTickCount
    DO
      GetCursorPos(p)
      ScreenToClient(Sprites.GameWindow, BYVAL VARPTR(p))
      Move_Paddle(p.Y)
    LOOP UNTIL GetTickCount-iTime>Performance

    Move_Ball

    Handle_DialogEvents

  LOOP UNTIL GameOver > 0
END SUB

'*****************************************************
'* Handle_DialogEvents                               *
'*                                                   *
'* Handle windows and dialog messages during the game*
'*****************************************************
FUNCTION Handle_DialogEvents AS LONG
  STATIC Msg AS tagMsg

  IF PeekMessage(Msg, %NULL, 0, 0, %PM_REMOVE) THEN
    TranslateMessage Msg
    DispatchMessage Msg
  END IF

  IF Player.OS = %VER_PLATFORM_WIN32_WINDOWS THEN
    SLEEP 1
  ELSE
    timeBeginPeriod(1)    ' Sets the minimum timer resolution and
    SLEEP 1               ' gives more accurate time on NT systems
    timeEndPeriod(1)      ' Do not need QueryPerformanceCounter for pong
  END IF

  Handle_DialogEvents = 1
END FUNCTION

'*****************************************************
'* Get_SysPerform                                    *
'*                                                   *
'* Calculate the systems performance level by doing  *
'* some sample graphics drawing and copy statements. *
'*                                                   *
'* Return Value:                                     *
'*      A LONG integer representing the systems      *
'*      performance level.                           *
'*****************************************************
FUNCTION Get_SysPerform AS LONG
  LOCAL sTime AS DWORD
  LOCAL i AS LONG
  LOCAL msg AS tagmsg

  PlaySound("AppStart.wav", BYVAL hModule, %SND_RESOURCE OR %SND_ASYNC)

  sTime = GetTickCount

  FOR i = 1 TO 50
    ' Set the game window as the current graphics window
    GRAPHIC ATTACH Sprites.GameWindow, 0

    ' Clear the graphics window with a random color
    GRAPHIC CLEAR RGB(RND(0, 255), RND(0, 255), RND(0,255))

    ' Set the paddles background bitmap as the current graphics window
    GRAPHIC ATTACH Sprites.PaddleBack, 0

    ' Copy a part of the game window's background into the paddle's background bitmap
    GRAPHIC COPY Sprites.GameWindow, 0, _
                 (%MIN_X, Paddle.Y) -  (%MIN_X+%PADDLE_WIDTH, Paddle.Y+%PADDLE_HEIGHT) TO (0, 0)

    ' Set the game window as the current graphics window
    GRAPHIC ATTACH Sprites.GameWindow, 0

    ' Copy the paddles bakcground bitmap to the current graphics window
    GRAPHIC COPY Sprites.PaddleBack, 0, _
                 (%MIN_X, Paddle.Y) -  (%MIN_X+%PADDLE_WIDTH, Paddle.Y+%PADDLE_HEIGHT) TO (0, 0)

    Handle_DialogEvents
   NEXT i

  sTime = GetTickCount - sTime

  SELECT CASE sTime
    CASE <75
      Performance = 9
      Paddle.YVelocity = 1
    CASE <250
      Performance = 7
      Paddle.YVelocity = 3
    CASE <500
      Performance = 7
      Paddle.YVelocity = 5
    CASE <1000
      Performance = 5
      Paddle.YVelocity = 7
    CASE ELSE
      Performance = 0
      Paddle.YVelocity = 9
  END SELECT
END FUNCTION



'*****************************************************
'* Get_OSVersion                                     *
'*                                                   *
'* Determine the players Operating System version.   *
'*                                                   *
'* Return Value:                                     *
'*      A LONG integer representing the players      *
'*      Operating System.                            *
'*****************************************************
FUNCTION Get_OSVersion AS LONG
  LOCAL OSVer AS OSVERSIONINFO

  OSVer.dwOSVersionInfoSize = SIZEOF(OSVer)
  GetVersionEX(OSVer)

  Get_OSVersion = OSVer.dwPlatformId
END FUNCTION
